package nz.ac.massey.programming_project_159333_s1_2021;
/**
 * ECS manager class.
 * Call this class and its functions to interact correctly with the ECS system.
 * 
 * Contributors:
 * Brychan Dempsey - brychand@hotmail.com
 * 
 * References:
 * Based on the implementation by Austin Morlan:
 * https://code.austinmorlan.com/austin/ecs - 'A simple C++ Entity Component System' - released under MIT licence
 *  
 */

import java.util.BitSet;
/**
 * The ECS manager.
 * <p>
 * See <href src="https://git.software.kauripeak.co.nz/BrychanD/JavaECS">https://git.software.kauripeak.co.nz/BrychanD/JavaECS</href>
 * for documentation and more information.
 */
public class ECS {
    protected EntityManager entityManager;
    protected ComponentManager componentManager;
    protected SystemManager systemManager;
    /**
     * Initialises the ECS with default values
     * <p>
     * Maximum 1024 enitites default
     */
    public ECS(){
        entityManager = new EntityManager();
        componentManager = new ComponentManager();
        systemManager = new SystemManager();
    }

    /**
     * Initialises the ECS with the specified value(s)
     * @param maxEntities the maximum number of entities to allow
     */
    public ECS(int maxEntities){
        entityManager = new EntityManager(maxEntities);
        componentManager = new ComponentManager();
        systemManager = new SystemManager();
    }

    /**
     * Creates a new entity
     * @return the id of the new entity
     */
    Integer createEntity(){
        return entityManager.addEntity();
    }

    /**
     * Signals each manager to remove the specified entity
     * @param entity the entity to destroy
     */
    void destroyEntity(int entity){
        entityManager.removeEntity(entity);
        componentManager.entityDestroyed(entity);
        systemManager.entityDestroyed(entity);
    }

    /**
     * Registers the specified name in the component manager
     * @param name the name to register. Should be the component class name or a suitable name for primitive types
     */
    void registerComponent(String name){
        componentManager.registerComponent(name);
    }

    Integer getComponentIndex(String name){
        return componentManager.getComponentIndex(name);
    }

    /**
     * Adds an exisiting component to an exisiting entity
     * @param entity the entity to add the component to
     * @param componentName the class name of the component to add
     * @param component the actual component data
     */
    void addComponent(int entity, String componentName, Object component){
        componentManager.addComponentToEntity(componentName, component, entity);
        entityManager.registerComponent(componentManager.getComponentIndex(componentName), entity);
        systemManager.entitySignatureChanged(entity, entityManager.getRegistrations(entity));
    }

    /**
     * Removes the component from the specified entity
     * @param entity the entity to remove the component from
     * @param componentName the class name of the component
     */
    void removeComponent(int entity, String componentName){
        componentManager.removeComponentFromEntity(componentName, entity);
        entityManager.unregisterComponent(componentManager.getComponentIndex(componentName), entity);
        systemManager.entitySignatureChanged(entity, entityManager.getRegistrations(entity));
    }

    /**
     * Gets the actual data of the component associated to the entity.
     * May require casting from Object to the known data type
     * @param entity the entity to retrieve the data for
     * @param componentName the class name of the component
     * @return the component data Object associated with the entity
     */
    Object getComponentData(int entity, String componentName){
        return componentManager.getComponent(componentName, entity);
    }

    /**
     * 
     * @param systemName
     * @param action
     */
    void registerSystem(String systemName, ECSSystem action){
        systemManager.registerSystem(systemName, action);
    }

    /**
     * Sets the specified system's signature to the provided signature
     * @param system the class name of the system to set the signature of
     * @param signature the new signature data
     */
    void setSystemSignature(String system, BitSet signature){
        systemManager.setSignature(system, signature);
    }
}